@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::exc::*;
-use crate::unicode::*;
+use crate::serialize::error::*;
+use crate::str::*;
 
 use serde::ser::{Serialize, Serializer};
 
@@ -16,17 +16,16 @@ impl StrSerializer {
     }
 }
 
-impl<'p> Serialize for StrSerializer {
+impl Serialize for StrSerializer {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let uni = read_utf8_from_str(self.ptr, &mut str_size);
-        if unlikely!(uni.is_null()) {
+        let uni = unicode_to_str(self.ptr);
+        if unlikely!(uni.is_none()) {
             err!(SerializeError::InvalidStr)
         }
-        serializer.serialize_str(str_from_slice!(uni, str_size))
+        serializer.serialize_str(uni.unwrap())
     }
 }
 
@@ -41,17 +40,16 @@ impl StrSubclassSerializer {
     }
 }
 
-impl<'p> Serialize for StrSubclassSerializer {
+impl Serialize for StrSubclassSerializer {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let uni = ffi!(PyUnicode_AsUTF8AndSize(self.ptr, &mut str_size)) as *const u8;
-        if unlikely!(uni.is_null()) {
+        let uni = unicode_to_str_via_ffi(self.ptr);
+        if unlikely!(uni.is_none()) {
             err!(SerializeError::InvalidStr)
         }
-        serializer.serialize_str(str_from_slice!(uni, str_size))
+        serializer.serialize_str(uni.unwrap())
     }
 }
