@@ -27,15 +27,15 @@ pub struct PyCompactUnicodeObject {
 }
 
 #[cfg(not(Py_3_12))]
-const STATE_ASCII: u32 = 0b00000000000000000000000001000000;
+const STATE_ASCII: u32 = 1 << 6;
 #[cfg(not(Py_3_12))]
-const STATE_COMPACT: u32 = 0b00000000000000000000000000100000;
+const STATE_COMPACT: u32 = 1 << 5;
 
 #[cfg(Py_3_12)]
-const STATE_ASCII: u32 = 0b00000000000000000000000000100000;
+const STATE_ASCII: u32 = 1 << 5;
 
 #[cfg(Py_3_12)]
-const STATE_COMPACT: u32 = 0b00000000000000000000000000010000;
+const STATE_COMPACT: u32 = 1 << 4;
 
 const STATE_COMPACT_ASCII: u32 = STATE_COMPACT | STATE_ASCII;
 
