@@ -1,10 +1,10 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use ahash::RandomState;
-use once_cell::unsync::Lazy;
+use once_cell::race::OnceBox;
 use pyo3_ffi::*;
 use std::os::raw::c_char;
-use std::ptr::NonNull;
+use std::ptr::{null_mut, NonNull};
 use std::sync::Once;
 
 pub struct NumpyTypes {
@@ -23,86 +23,97 @@ pub struct NumpyTypes {
     pub datetime64: *mut PyTypeObject,
 }
 
-pub static mut DEFAULT: *mut PyObject = 0 as *mut PyObject;
-pub static mut OPTION: *mut PyObject = 0 as *mut PyObject;
+pub static mut DEFAULT: *mut PyObject = null_mut();
+pub static mut OPTION: *mut PyObject = null_mut();
 
-pub static mut NONE: *mut PyObject = 0 as *mut PyObject;
-pub static mut TRUE: *mut PyObject = 0 as *mut PyObject;
-pub static mut FALSE: *mut PyObject = 0 as *mut PyObject;
+pub static mut NONE: *mut PyObject = null_mut();
+pub static mut TRUE: *mut PyObject = null_mut();
+pub static mut FALSE: *mut PyObject = null_mut();
+pub static mut EMPTY_UNICODE: *mut PyObject = null_mut();
 
-pub static mut BYTES_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut BYTEARRAY_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut MEMORYVIEW_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut STR_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut INT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut BOOL_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut NONE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut FLOAT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut LIST_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DICT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DATETIME_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DATE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut TIME_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut TUPLE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut UUID_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut ENUM_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
+pub static mut BYTES_TYPE: *mut PyTypeObject = null_mut();
+pub static mut BYTEARRAY_TYPE: *mut PyTypeObject = null_mut();
+pub static mut MEMORYVIEW_TYPE: *mut PyTypeObject = null_mut();
+pub static mut STR_TYPE: *mut PyTypeObject = null_mut();
+pub static mut INT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut BOOL_TYPE: *mut PyTypeObject = null_mut();
+pub static mut NONE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut FLOAT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut LIST_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DICT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DATETIME_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DATE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut TIME_TYPE: *mut PyTypeObject = null_mut();
+pub static mut TUPLE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut UUID_TYPE: *mut PyTypeObject = null_mut();
+pub static mut ENUM_TYPE: *mut PyTypeObject = null_mut();
+pub static mut FIELD_TYPE: *mut PyTypeObject = null_mut();
+
+pub static mut NUMPY_TYPES: OnceBox<Option<NonNull<NumpyTypes>>> = OnceBox::new();
 
 #[cfg(Py_3_9)]
-pub static mut ZONEINFO_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
+pub static mut ZONEINFO_TYPE: *mut PyTypeObject = null_mut();
 
-pub static mut NUMPY_TYPES: Lazy<Option<NumpyTypes>> = Lazy::new(|| unsafe { load_numpy_types() });
-pub static mut FIELD_TYPE: Lazy<NonNull<PyObject>> = Lazy::new(|| unsafe { look_up_field_type() });
+pub static mut UTCOFFSET_METHOD_STR: *mut PyObject = null_mut();
+pub static mut NORMALIZE_METHOD_STR: *mut PyObject = null_mut();
+pub static mut CONVERT_METHOD_STR: *mut PyObject = null_mut();
+pub static mut DST_STR: *mut PyObject = null_mut();
 
-pub static mut INT_ATTR_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut UTCOFFSET_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut NORMALIZE_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut CONVERT_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut EMPTY_UNICODE: *mut PyObject = 0 as *mut PyObject;
-pub static mut DST_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DICT_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DATACLASS_FIELDS_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut SLOTS_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut FIELD_TYPE_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut ARRAY_STRUCT_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DTYPE_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DESCR_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut VALUE_STR: *mut PyObject = 0 as *mut PyObject;
+pub static mut DICT_STR: *mut PyObject = null_mut();
+pub static mut DATACLASS_FIELDS_STR: *mut PyObject = null_mut();
+pub static mut SLOTS_STR: *mut PyObject = null_mut();
+pub static mut FIELD_TYPE_STR: *mut PyObject = null_mut();
+pub static mut ARRAY_STRUCT_STR: *mut PyObject = null_mut();
+pub static mut DTYPE_STR: *mut PyObject = null_mut();
+pub static mut DESCR_STR: *mut PyObject = null_mut();
+pub static mut VALUE_STR: *mut PyObject = null_mut();
+pub static mut INT_ATTR_STR: *mut PyObject = null_mut();
+
+pub static mut HASH_BUILDER: OnceBox<ahash::RandomState> = OnceBox::new();
+
+pub fn ahash_init() -> Box<ahash::RandomState> {
+    unsafe {
+        Box::new(RandomState::with_seeds(
+            VALUE_STR as u64,
+            DICT_TYPE as u64,
+            STR_TYPE as u64,
+            BYTES_TYPE as u64,
+        ))
+    }
+}
 
-pub static mut HASH_BUILDER: Lazy<ahash::RandomState> = Lazy::new(|| unsafe {
-    RandomState::with_seeds(
-        VALUE_STR as u64,
-        DICT_TYPE as u64,
-        STR_TYPE as u64,
-        BYTES_TYPE as u64,
-    )
-});
 #[cfg(feature = "yyjson")]
 pub const YYJSON_BUFFER_SIZE: usize = 1024 * 1024 * 8;
 
 #[cfg(feature = "yyjson")]
-pub static mut YYJSON_ALLOC: Lazy<crate::yyjson::yyjson_alc> = Lazy::new(|| unsafe {
-    let buffer = std::alloc::alloc(std::alloc::Layout::from_size_align_unchecked(
-        YYJSON_BUFFER_SIZE,
-        64,
-    ));
-    let mut alloc = crate::yyjson::yyjson_alc {
-        malloc: None,
-        realloc: None,
-        free: None,
-        ctx: std::ptr::null_mut(),
-    };
-    crate::yyjson::yyjson_alc_pool_init(
-        &mut alloc,
-        buffer as *mut std::os::raw::c_void,
-        YYJSON_BUFFER_SIZE,
-    );
-    alloc
-});
+pub static mut YYJSON_ALLOC: OnceBox<crate::yyjson::yyjson_alc> = OnceBox::new();
+
+#[cfg(feature = "yyjson")]
+pub fn yyjson_init() -> Box<crate::yyjson::yyjson_alc> {
+    unsafe {
+        let buffer = std::alloc::alloc(std::alloc::Layout::from_size_align_unchecked(
+            YYJSON_BUFFER_SIZE,
+            64,
+        ));
+        let mut alloc = crate::yyjson::yyjson_alc {
+            malloc: None,
+            realloc: None,
+            free: None,
+            ctx: null_mut(),
+        };
+        crate::yyjson::yyjson_alc_pool_init(
+            &mut alloc,
+            buffer as *mut std::os::raw::c_void,
+            YYJSON_BUFFER_SIZE,
+        );
+        Box::new(alloc)
+    }
+}
 
 #[allow(non_upper_case_globals)]
-pub static mut JsonEncodeError: *mut PyObject = 0 as *mut PyObject;
+pub static mut JsonEncodeError: *mut PyObject = null_mut();
 #[allow(non_upper_case_globals)]
-pub static mut JsonDecodeError: *mut PyObject = 0 as *mut PyObject;
+pub static mut JsonDecodeError: *mut PyObject = null_mut();
 
 static INIT: Once = Once::new();
 
@@ -143,6 +154,7 @@ pub fn init_typerefs() {
         TIME_TYPE = look_up_time_type();
         UUID_TYPE = look_up_uuid_type();
         ENUM_TYPE = look_up_enum_type();
+        FIELD_TYPE = look_up_field_type();
 
         #[cfg(Py_3_9)]
         {
@@ -176,13 +188,13 @@ pub fn init_typerefs() {
 #[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_json_exc() -> *mut PyObject {
     let module = PyImport_ImportModule("json\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "JSONDecodeError\0".as_ptr() as *const c_char)
         as *mut PyObject;
     let res = pyo3_ffi::PyErr_NewException(
         "orjson.JSONDecodeError\0".as_ptr() as *const c_char,
         ptr,
-        std::ptr::null_mut(),
+        null_mut(),
     );
     Py_DECREF(ptr);
     Py_DECREF(module_dict);
@@ -194,7 +206,7 @@ unsafe fn look_up_json_exc() -> *mut PyObject {
 #[cold]
 #[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut PyTypeObject {
-    let mod_dict = PyObject_GenericGetDict(numpy_module, std::ptr::null_mut());
+    let mod_dict = PyObject_GenericGetDict(numpy_module, null_mut());
     let ptr = PyMapping_GetItemString(mod_dict, np_type.as_ptr() as *const c_char);
     Py_XDECREF(ptr);
     Py_XDECREF(mod_dict);
@@ -203,49 +215,50 @@ unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut
 
 #[cold]
 #[cfg_attr(feature = "optimize", optimize(size))]
-unsafe fn load_numpy_types() -> Option<NumpyTypes> {
-    let numpy = PyImport_ImportModule("numpy\0".as_ptr() as *const c_char);
-    if numpy.is_null() {
-        PyErr_Clear();
-        return None;
+pub fn load_numpy_types() -> Box<Option<NonNull<NumpyTypes>>> {
+    unsafe {
+        let numpy = PyImport_ImportModule("numpy\0".as_ptr() as *const c_char);
+        if numpy.is_null() {
+            PyErr_Clear();
+            return Box::new(None);
+        }
+        let types = Box::new(NumpyTypes {
+            array: look_up_numpy_type(numpy, "ndarray\0"),
+            float32: look_up_numpy_type(numpy, "float32\0"),
+            float64: look_up_numpy_type(numpy, "float64\0"),
+            int8: look_up_numpy_type(numpy, "int8\0"),
+            int16: look_up_numpy_type(numpy, "int16\0"),
+            int32: look_up_numpy_type(numpy, "int32\0"),
+            int64: look_up_numpy_type(numpy, "int64\0"),
+            uint16: look_up_numpy_type(numpy, "uint16\0"),
+            uint32: look_up_numpy_type(numpy, "uint32\0"),
+            uint64: look_up_numpy_type(numpy, "uint64\0"),
+            uint8: look_up_numpy_type(numpy, "uint8\0"),
+            bool_: look_up_numpy_type(numpy, "bool_\0"),
+            datetime64: look_up_numpy_type(numpy, "datetime64\0"),
+        });
+        Py_XDECREF(numpy);
+        Box::new(Some(nonnull!(Box::<NumpyTypes>::into_raw(types))))
     }
-
-    let types = Some(NumpyTypes {
-        array: look_up_numpy_type(numpy, "ndarray\0"),
-        float32: look_up_numpy_type(numpy, "float32\0"),
-        float64: look_up_numpy_type(numpy, "float64\0"),
-        int8: look_up_numpy_type(numpy, "int8\0"),
-        int16: look_up_numpy_type(numpy, "int16\0"),
-        int32: look_up_numpy_type(numpy, "int32\0"),
-        int64: look_up_numpy_type(numpy, "int64\0"),
-        uint16: look_up_numpy_type(numpy, "uint16\0"),
-        uint32: look_up_numpy_type(numpy, "uint32\0"),
-        uint64: look_up_numpy_type(numpy, "uint64\0"),
-        uint8: look_up_numpy_type(numpy, "uint8\0"),
-        bool_: look_up_numpy_type(numpy, "bool_\0"),
-        datetime64: look_up_numpy_type(numpy, "datetime64\0"),
-    });
-    Py_XDECREF(numpy);
-    types
 }
 
 #[cold]
 #[cfg_attr(feature = "optimize", optimize(size))]
-unsafe fn look_up_field_type() -> NonNull<PyObject> {
+unsafe fn look_up_field_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("dataclasses\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "_FIELD\0".as_ptr() as *const c_char)
         as *mut PyTypeObject;
     Py_DECREF(module_dict);
     Py_DECREF(module);
-    NonNull::new_unchecked(ptr as *mut PyObject)
+    ptr
 }
 
 #[cold]
 #[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_enum_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("enum\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "EnumMeta\0".as_ptr() as *const c_char)
         as *mut PyTypeObject;
     Py_DECREF(module_dict);
@@ -257,7 +270,7 @@ unsafe fn look_up_enum_type() -> *mut PyTypeObject {
 #[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_uuid_type() -> *mut PyTypeObject {
     let uuid_mod = PyImport_ImportModule("uuid\0".as_ptr() as *const c_char);
-    let uuid_mod_dict = PyObject_GenericGetDict(uuid_mod, std::ptr::null_mut());
+    let uuid_mod_dict = PyObject_GenericGetDict(uuid_mod, null_mut());
     let uuid = PyMapping_GetItemString(uuid_mod_dict, "NAMESPACE_DNS\0".as_ptr() as *const c_char);
     let ptr = (*uuid).ob_type;
     Py_DECREF(uuid);
@@ -308,7 +321,7 @@ unsafe fn look_up_time_type() -> *mut PyTypeObject {
 #[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_zoneinfo_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("zoneinfo\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "ZoneInfo\0".as_ptr() as *const c_char)
         as *mut PyTypeObject;
     Py_DECREF(module_dict);
